--- Status: 03.07.2017

--- combine TravisTorrent and GHTorrent to get information about projects
select
  tr_projects.gh_project_name as gh_project_name,
  gh_projects.gh_owner_id as gh_owner_id,
  gh_projects.gh_project_id as gh_project_id,
  tr_projects.tr_first_build as tr_first_build,
  tr_projects.tr_last_build as tr_last_build,
  tr_projects.tr_timespan as tr_timespan_during_ci,
  tr_projects.tr_build_count as tr_build_count
from
  (select gh_project_name,
    tr_first_build,
    tr_last_build,
    DATEDIFF(tr_last_build, tr_first_build) as tr_timespan,
    tr_build_count
  from
    (select gh_project_name,
      MIN(TIMESTAMP(gh_build_started_at)) as tr_first_build,
      MAX(TIMESTAMP(gh_build_started_at)) as tr_last_build,
      COUNT(tr_build_id) as tr_build_count
    from [travistorrent.travistorrent_11_1_2017]
    group by gh_project_name)
  ) as tr_projects
join
  (select projects.owner_id as gh_owner_id,
    projects.id as gh_project_id,
    CONCAT(users.login, "/", projects.name) as gh_project_name
  from [ghtorrent-bq:ght.projects] projects
  join [ghtorrent-bq:ght.users] users
  on projects.owner_id = users.id
  ) as gh_projects
on tr_projects.gh_project_name = gh_projects.gh_project_name
order by tr_timespan desc;

=> tr_projects

--- get first commit, last commit, and commit count for projects
select
  tr_project_commits.project_id as gh_project_id,
  count(gh_commits.id) as gh_commit_count,
  min(timestamp(gh_commits.created_at)) as gh_first_commit,
  max(timestamp(gh_commits.created_at)) as gh_last_commit
from
  (select
    tr_projects.gh_project_id as project_id,
    gh_project_commits.commit_id as commit_id
  from [travistorrent.tr_projects] as tr_projects
  join [ghtorrent-bq:ght.project_commits] as gh_project_commits
  on tr_projects.gh_project_id = gh_project_commits.project_id
  ) as tr_project_commits
join [ghtorrent-bq:ght.commits] gh_commits
on tr_project_commits.commit_id = gh_commits.id
where gh_commits.created_at>0  # some timestamps are broken
group by gh_project_id;

=> gh_projects_commits

--- merge tables
SELECT
  tr_projects.gh_project_name as gh_project_name,
  tr_projects.gh_owner_id as gh_owner_id,
  tr_projects.gh_project_id as gh_project_id,
  tr_projects.tr_first_build as tr_first_build,
  tr_projects.tr_last_build as tr_last_build,
  tr_projects.tr_build_count as tr_build_count,
  tr_projects.tr_timespan_during_ci as tr_timespan_during_ci,
  gh_projects_commits.gh_commit_count as gh_commit_count,
  gh_projects_commits.gh_first_commit as gh_first_commit,
  gh_projects_commits.gh_last_commit as gh_last_commit
FROM [msr-mining-challenge:travistorrent.tr_projects] as tr_projects
JOIN [msr-mining-challenge:travistorrent.gh_projects_commits] as gh_projects_commits
ON tr_projects.gh_project_id = gh_projects_commits.gh_project_id;

=> tr_projects_commits

--- count commits before, during, and after travisci build
SELECT
  project_id as gh_project_id,
  sum(gh_commit_count_before_first_build) as gh_commit_count_before_ci,
  sum(gh_commit_count_during_ci) as gh_commit_count_during_ci,
  sum(gh_commit_count_after_last_build) as gh_commit_count_after_ci
FROM
  (SELECT
    gh_project_commits.project_id as project_id,
    CASE WHEN DATEDIFF(gh_project_commits.tr_first_build, gh_commits.created_at) > 0 THEN 1 ELSE 0 END
      AS gh_commit_count_before_first_build,
    CASE WHEN DATEDIFF (gh_project_commits.tr_first_build, gh_commits.created_at) <= 0 and DATEDIFF(gh_project_commits.tr_last_build, gh_commits.created_at) >= 0 THEN 1 ELSE 0 END
      AS gh_commit_count_during_ci,
    CASE WHEN DATEDIFF(gh_project_commits.tr_last_build, gh_commits.created_at) < 0 THEN 1 ELSE 0 END
      AS gh_commit_count_after_last_build
  FROM
    (SELECT
        tr_projects.gh_project_id as project_id,
        tr_projects.tr_first_build as tr_first_build,
        tr_projects.tr_last_build as tr_last_build,
        gh_project_commits.commit_id as commit_id
      FROM [travistorrent.tr_projects] as tr_projects
      JOIN  [ghtorrent-bq:ght.project_commits] as gh_project_commits
      ON tr_projects.gh_project_id = gh_project_commits.project_id)
      AS gh_project_commits
  JOIN [ghtorrent-bq:ght.commits] gh_commits
  ON gh_project_commits.commit_id = gh_commits.id)
group by gh_project_id;

=> gh_projects_commits_time

--- merge tables
SELECT
  tr_projects_commits.gh_project_name as gh_project_name,
  tr_projects_commits.gh_owner_id as gh_owner_id,
  tr_projects_commits.gh_project_id as gh_project_id,
  tr_projects_commits.tr_first_build as tr_first_build,
  tr_projects_commits.tr_last_build as tr_last_build,
  tr_projects_commits.gh_first_commit as gh_first_commit,
  tr_projects_commits.gh_last_commit as gh_last_commit,
  CASE WHEN DATEDIFF(tr_projects_commits.tr_first_build, tr_projects_commits.gh_first_commit)>=0
    THEN DATEDIFF(tr_projects_commits.tr_first_build, tr_projects_commits.gh_first_commit) ELSE 0 END
    AS tr_timespan_before_ci,
  tr_projects_commits.tr_timespan_during_ci as tr_timespan_during_ci,
  CASE WHEN DATEDIFF(tr_projects_commits.gh_last_commit, tr_projects_commits.tr_last_build)>=0
    THEN DATEDIFF(tr_projects_commits.gh_last_commit, tr_projects_commits.tr_last_build) ELSE 0 END
    AS tr_timespan_after_ci,
  tr_projects_commits.tr_build_count as tr_build_count,
  tr_projects_commits.gh_commit_count as gh_commit_count,
  gh_projects_commits_time.gh_commit_count_before_ci as gh_commit_count_before_ci,
  gh_projects_commits_time.gh_commit_count_during_ci as gh_commit_count_during_ci,
  gh_projects_commits_time.gh_commit_count_after_ci as gh_commit_count_after_ci
FROM [msr-mining-challenge:travistorrent.tr_projects_commits] as tr_projects_commits
JOIN [msr-mining-challenge:travistorrent.gh_projects_commits_time] as gh_projects_commits_time
ON tr_projects_commits.gh_project_id = gh_projects_commits_time.gh_project_id
ORDER BY gh_commit_count_during_ci DESC;

=> tr_projects_commits_time
