setwd("/Users/sebastian/git/projects/ci/data-retrieval")
#setwd("F:/Git/projects/ci/data-retrieval")

# load global function
source("../global.R")

# load local functions
source("functions.R")

# read project information
data_commits <- read.csv("3_commit_hash_count_branches.csv", header=TRUE, sep=",", quote="\"", na.strings=c(""), stringsAsFactors=FALSE)
data_merges <- read.csv("3_merge_hash_count_branches.csv", header=TRUE, sep=",", quote="\"", na.strings=c(""), stringsAsFactors=FALSE)

# parse boolean columns
data_commits$is_default <- as.logical(data_commits$is_default)
data_merges$is_default <- as.logical(data_merges$is_default)

# filter default branches
data_commits <- data_commits[data_commits$is_default == TRUE,]
nrow(data_commits)
# 544
data_merges <- data_merges[data_merges$is_default == TRUE,]
nrow(data_merges)
# 542

# recalculate ratio
data_commits$ratio <- data_commits$commit_hash_count/data_commits$total_commit_hash_count
data_merges$ratio <- data_merges$merge_hash_count/data_merges$total_merge_hash_count

boxplot(data_commits$ratio, data_merges$ratio)

d <- density(data_commits$ratio)
plot(d)
d <- density(data_merges$ratio)
plot(d)

summary(data_commits$ratio)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 0.0490  0.9506  0.9886  0.9476  1.0000  1.0000 
summary(data_merges$ratio)
# Min. 1st Qu.  Median    Mean 3rd Qu.    Max. 
# 0.0303  0.9138  0.9870  0.9250  1.0000  1.0000 

thresholds_commits <- outlier_thresholds(data_commits$ratio, 1.5)
thresholds_commits$lower
# 0.8765751
thresholds_commits$upper
# 1.074055 > 1.0, no upper outliers

thresholds_merges <- outlier_thresholds(data_merges$ratio, 1.5)
thresholds_merges$lower
# 0.7843915
thresholds_merges$upper
# 1.129365 > 1.0, no upper outliers

nrow(data_commits)
# 544
nrow(data_commits[data_commits$ratio>=thresholds_commits$lower,])
# 479

nrow(data_merges)
# 542
nrow(data_merges[data_merges$ratio>=thresholds_merges$lower,])
# 485

# merge commits and merges
data <- data.frame(data_commits$project,
                   data_commits$branch,
                   data_commits$is_default,
                   data_commits$commit_hash_count,
                   data_commits$total_commit_hash_count
)
names(data) <- c("project",
                 "branch",
                 "is_default",
                 "commit_hash_count",
                 "total_commit_hash_count"
)
data <- merge(data,
              data_merges[,c("project", "branch", "merge_hash_count", "total_merge_hash_count")],
              by=c("project", "branch"),
              all.x=TRUE
)
# replaces NA with 0 (for addition below)
data$commit_hash_count[is.na(data$commit_hash_count)] <- 0
data$total_commit_hash_count[is.na(data$total_commit_hash_count)] <- 0
data$merge_hash_count[is.na(data$merge_hash_count)] <- 0
data$total_merge_hash_count[is.na(data$total_merge_hash_count)] <- 0
# get total counts (commits+merges)
data$hash_count <- data$commit_hash_count + data$merge_hash_count
data$total_hash_count <- data$total_commit_hash_count + data$total_merge_hash_count
# get ratio
data$ratio <- data$hash_count/data$total_hash_count

# outliers
thresholds <- outlier_thresholds(data$ratio, 1.5)
thresholds$lower
# 0.8560787
thresholds$upper
# 1.085258 > 1.0, no upper outliers

nrow(data)
# 544
nrow(data[data$ratio>=thresholds$lower,])
# 487

# DECISION: Select projects with an ratio of >= 0.8560787 (exclude outliers)

# filter data
data_filtered <- data[data$ratio>=thresholds$lower,]
nrow(data_filtered)
# 487

# create figures
plot_data(data_commits,
          thresholds_commits,
          "3_commit_hash_count_branches.pdf",
          "Percentage of commits on default branch",
          "Commits on default branch (%)"
)
plot_data(data_merges,
          thresholds_merges,
          "3_merge_hash_count_branches.pdf",
          "Percentage of merges on default branch",
          "Merges on default branch (%)"
)
plot_data(data,
          thresholds,
          "3_hash_count_branches.pdf",
          "Percentage of commits and merges on default branch",
          "Commits and merges on default branch (%)"
)

# export filtered projects
write.table(data_filtered$project, file="3_projects_filtered_commit_count.csv", sep=";", col.names=FALSE, row.names=FALSE, na="", quote=TRUE, qmethod="double", fileEncoding="UTF-8")


