--- Status: 18.07.2017

--- analyze proportion of commits to default branch
--- (commits may also be present in other branches an later merged into default branch)
WITH
  total_commit_count AS (
    SELECT project, COUNT(DISTINCT hash_value) as commit_hash_count
    FROM `project_sample.tr_sample_commits`
    GROUP BY project
  ),
  branch_commit_count AS (
    SELECT project, branch, COUNT(DISTINCT hash_value) as commit_hash_count
    FROM `project_sample.tr_sample_commits`
    GROUP BY project, branch
  ),
  default_branch AS (
    SELECT project, branch
    FROM `project_sample.tr_sample_branches`
    WHERE default_branch=true
  )
SELECT
  data.project as project,
  data.branch as branch,
  is_default,
  data.commit_hash_count as commit_hash_count,
  total_commit_count.commit_hash_count as total_commit_hash_count,
  data.commit_hash_count/total_commit_count.commit_hash_count as ratio
FROM (  
  SELECT
    branch_commit_count.project as project,
    branch_commit_count.branch as branch,
    commit_hash_count,
    CASE
      WHEN default_branch.branch IS NULL
      THEN false
      ELSE true
    END as is_default
  FROM branch_commit_count
  LEFT JOIN default_branch
  ON branch_commit_count.project = default_branch.project
    AND branch_commit_count.branch = default_branch.branch
) as data 
LEFT JOIN total_commit_count
ON data.project = total_commit_count.project
ORDER BY project, commit_hash_count DESC, branch;

=> 3_commit_hash_count_branches.csv


--- analyze proportion of merges into default branch
--- (commits may also be present in other branches an later merged into default branch)
WITH
  total_merge_count AS (
    SELECT project, COUNT(DISTINCT hash_value) as merge_hash_count
    FROM `project_sample.tr_sample_merges`
    GROUP BY project
  ),
  branch_merge_count AS (
    SELECT project, branch, COUNT(DISTINCT hash_value) as merge_hash_count
    FROM `project_sample.tr_sample_merges`
    GROUP BY project, branch
  ),
  default_branch AS (
    SELECT project, branch
    FROM `project_sample.tr_sample_branches`
    WHERE default_branch=true
  )
SELECT
  data.project as project,
  data.branch as branch,
  is_default,
  data.merge_hash_count as merge_hash_count,
  total_merge_count.merge_hash_count as total_merge_hash_count,
  data.merge_hash_count/total_merge_count.merge_hash_count as ratio
FROM (  
  SELECT
    branch_merge_count.project as project,
    branch_merge_count.branch as branch,
    merge_hash_count,
    CASE
      WHEN default_branch.branch IS NULL
      THEN false
      ELSE true
    END as is_default
  FROM branch_merge_count
  LEFT JOIN default_branch
  ON branch_merge_count.project = default_branch.project
    AND branch_merge_count.branch = default_branch.branch
) as data 
LEFT JOIN total_merge_count
ON data.project = total_merge_count.project
ORDER BY project, merge_hash_count DESC, branch;

=> 3_merge_hash_count_branches.csv

--- Analyze distribution using R script.
--- Afterwards: Import "3_projects_filtered_commit_count.csv" into tr_projects_filtered

--- check if all projects have been successfully imported
SELECT COUNT(DISTINCT project)
FROM `project_sample.tr_projects_filtered`;
--- 487

SELECT
  c.project as project, branch, hash_value, author_name, author_email, author_date,
  commit_name, commit_email, commit_date, log_message_length,
  file_count, lines_added, lines_deleted, file_extensions
FROM `project_sample.tr_sample_commits` c
INNER JOIN `project_sample.tr_projects_filtered` f
ON c.project = f.project;
=> project_sample.tr_sample_commits_filtered

SELECT COUNT(DISTINCT project)
FROM `project_sample.tr_sample_commits_filtered`;
--- 487

SELECT
  m.project as project, branch, hash_value, merged_commits, author_name, author_email, author_date,
  commit_name, commit_email, commit_date, log_message_length,
  file_count, lines_added, lines_deleted, file_extensions,
  pull_request_id, source_user, source_branch
FROM `project_sample.tr_sample_merges` m
INNER JOIN `project_sample.tr_projects_filtered` f
ON m.project = f.project;
=> project_sample.tr_sample_merges_filtered

SELECT COUNT(DISTINCT project)
FROM `project_sample.tr_sample_merges_filtered`;
--- 485


----------------------------------------------------------------------
--- export from here on into dataset project_sample_default_branch ---
----------------------------------------------------------------------

--- filter commits to default branch 
SELECT
  c.project as project,
  c.branch as branch,
  hash_value,
  author_name,
  author_email,
  author_date,
  commit_name,
  commit_email,
  commit_date,
  log_message_length,
  file_count,
  lines_added,
  lines_deleted,
  file_extensions
FROM `project_sample.tr_sample_commits_filtered` as c
JOIN (
  SELECT project, branch as default_branch
  FROM `project_sample.tr_sample_branches`
  WHERE default_branch=true
) as b
ON c.project = b.project
  AND c.branch = b.default_branch;
=> project_sample_default_branch.tr_sample_commits_default_branch


--- filter merges into default branch 
SELECT
  m.project as project,
  m.branch as branch,
  hash_value,
  merged_commits,
  author_name,
  author_email,
  author_date,
  commit_name,
  commit_email,
  commit_date,
  log_message_length,
  file_count,
  lines_added,
  lines_deleted,
  file_extensions,
  pull_request_id,
  source_user,
  source_branch
FROM `project_sample.tr_sample_merges_filtered` as m
JOIN (
  SELECT project, branch as default_branch
  FROM `project_sample.tr_sample_branches`
  WHERE default_branch=true
) as b
ON m.project = b.project
  AND m.branch = b.default_branch;
=> project_sample_default_branch.tr_sample_merges_default_branch


--- get first/last commit/merge to default branch and first/last build for default branch
WITH
  commit_date AS (
    SELECT
      project,
      branch,
      MIN(commit_date) as min_commit_date,
      MAX(commit_date) as max_commit_date
    FROM
      `project_sample_default_branch.tr_sample_commits_default_branch`
    GROUP BY project, branch
  ),
  merge_date AS (
    SELECT
      project,
      branch,
      MIN(commit_date) as min_merge_date,
      MAX(commit_date) as max_merge_date
    FROM
      `project_sample_default_branch.tr_sample_merges_default_branch`
    GROUP BY project, branch
  ),
  build_dates AS (
    SELECT
      project,
      default_branch,
      git_branch,
      MIN(gh_build_started_at) AS first_build,
      MAX(gh_build_started_at) AS last_build,
      COUNT(DISTINCT tr_build_id) as build_count
    FROM (
      SELECT project, branch as default_branch
      FROM `project_sample.tr_sample_branches`
      WHERE default_branch = true
    ) as branches
    LEFT JOIN
      `travistorrent.tr_projects_sample_builds` as builds 
    ON branches.project = REPLACE(builds.gh_project_name, '/', '_') AND (branches.default_branch = builds.git_branch)
    GROUP BY project, default_branch, git_branch
  )
SELECT
  activity.project as project,
  activity.default_branch as default_branch,
  CAST(min_activity_date AS DATETIME) as min_activity_date,
  CAST(max_activity_date AS DATETIME) as max_activity_date,
  first_build,
  last_build,
  build_count,
  CAST(min_commit_date AS DATETIME) as min_commit_date,
  CAST(max_commit_date AS DATETIME) as max_commit_date,
  CAST(min_merge_date AS DATETIME) as min_merge_date,
  CAST(max_merge_date AS DATETIME) as max_merge_date  
FROM (
  SELECT
    commit_date.project as project,
    commit_date.branch as default_branch,
    CASE
      WHEN min_commit_date IS NULL THEN min_merge_date
      WHEN min_merge_date IS NULL THEN min_commit_date
      WHEN min_commit_date<min_merge_date THEN min_commit_date
      ELSE min_merge_date
    END as min_activity_date,
    CASE
      WHEN max_commit_date IS NULL THEN max_merge_date
      WHEN max_merge_date IS NULL THEN max_commit_date
      WHEN max_commit_date>min_merge_date THEN max_commit_date
      ELSE max_merge_date
    END as max_activity_date,
    min_commit_date,
    max_commit_date,
    min_merge_date,
    max_merge_date
  FROM commit_date
  LEFT JOIN merge_date
  ON commit_date.project = merge_date.project
    AND commit_date.branch = merge_date.branch
) as activity
LEFT JOIN build_dates
ON activity.project = build_dates.project
  AND activity.default_branch = build_dates.default_branch;
=> project_sample_default_branch.tr_sample_default_branch_activity


--- get projects where last build is more than one week newer than last commit/merge activity on default branch
SELECT project, DATE_DIFF(CAST(last_build AS DATE), CAST(max_activity_date AS DATE), DAY) as last_build_vs_activity
FROM `project_sample_default_branch.tr_sample_default_branch_activity`;
=> 3_last_build_vs_activity.csv

--- Analyze distribution using R script.
--- Afterwards: Import "3_projects_filtered_build_activity.csv" into tr_projects_filtered

--- check if all projects have been successfully imported
SELECT COUNT(DISTINCT project)
FROM `project_sample.tr_projects_filtered`;
--- 324

SELECT
  c.project as project, branch, hash_value, author_name, author_email, author_date,
  commit_name, commit_email, commit_date, log_message_length,
  file_count, lines_added, lines_deleted, file_extensions
FROM `project_sample.tr_sample_commits` c
INNER JOIN `project_sample.tr_projects_filtered` f
ON c.project = f.project;
=> project_sample.tr_sample_commits_filtered

SELECT COUNT(DISTINCT project)
FROM `project_sample.tr_sample_commits_filtered`;
--- 324

SELECT
  m.project as project, branch, hash_value, merged_commits, author_name, author_email, author_date,
  commit_name, commit_email, commit_date, log_message_length,
  file_count, lines_added, lines_deleted, file_extensions,
  pull_request_id, source_user, source_branch
FROM `project_sample.tr_sample_merges` m
INNER JOIN `project_sample.tr_projects_filtered` f
ON m.project = f.project;
=> project_sample.tr_sample_merges_filtered

SELECT COUNT(DISTINCT project)
FROM `project_sample.tr_sample_merges_filtered`;
--- 322

--- filter activity table
SELECT
  a.project as project,
  default_branch,
  min_activity_date,
  max_activity_date,
  first_build,
  last_build,
  build_count,
  min_commit_date,
  max_commit_date,
  min_merge_date,
  max_merge_date  
FROM `project_sample_default_branch.tr_sample_default_branch_activity` a
INNER JOIN `project_sample.tr_projects_filtered` f
ON a.project = f.project;
=> project_sample_default_branch.tr_sample_default_branch_activity_filtered

--- check filtering
SELECT COUNT(*)
FROM `project_sample_default_branch.tr_sample_default_branch_activity_filtered`;
=> 324


--- check for projects without commit, merge, or build info on default branch
SELECT *
FROM `project_sample_default_branch.tr_sample_default_branch_activity_filtered`
WHERE min_activity_date is null
  OR max_activity_date is null
  OR first_build is null
  OR last_build is null;

--- none found
 
 
--- get projects where first build is older than first commit/merge activity on default branch
SELECT *
FROM `project_sample_default_branch.tr_sample_default_branch_activity_filtered`
WHERE DATE_DIFF(CAST(min_activity_date AS DATE), CAST(first_build AS DATE), DAY) > 0;

--- none found


--- filter commits to default branch
SELECT
  commits.project as project,
  branch,
  hash_value,
  author_name,
  author_email,
  author_date,
  commit_name,
  commit_email,
  commit_date,
  log_message_length,
  file_count,
  lines_added,
  lines_deleted,
  file_extensions
FROM `project_sample_default_branch.tr_sample_commits_default_branch` commits
INNER JOIN (
  SELECT DISTINCT project
  FROM `project_sample_default_branch.tr_sample_default_branch_activity_filtered`
) filtered_projects
ON commits.project = filtered_projects.project;
=> project_sample_default_branch.tr_sample_commits_default_branch_filtered


--- filter merges into default branch
SELECT
  merges.project as project,
  branch,
  hash_value,
  merged_commits,
  author_name,
  author_email,
  author_date,
  commit_name,
  commit_email,
  commit_date,
  log_message_length,
  file_count,
  lines_added,
  lines_deleted,
  file_extensions,
  pull_request_id,
  source_user,
  source_branch
FROM `project_sample_default_branch.tr_sample_merges_default_branch` merges
INNER JOIN (
  SELECT DISTINCT project
  FROM `project_sample_default_branch.tr_sample_default_branch_activity_filtered`
) filtered_projects
ON merges.project = filtered_projects.project;
=> project_sample_default_branch.tr_sample_merges_default_branch_filtered


--- number of remaining projects
SELECT COUNT(DISTINCT project) FROM `project_sample_default_branch.tr_sample_default_branch_activity_filtered`;
--- 324
SELECT COUNT(*) FROM (
  SELECT DISTINCT project FROM `project_sample_default_branch.tr_sample_commits_default_branch_filtered`
  UNION DISTINCT
  SELECT DISTINCT project FROM `project_sample_default_branch.tr_sample_merges_default_branch_filtered`
);
--- 324


--- analyze how many commits on the default branch are also present in another branch (branch_count = 1 -> commit directly to default branch?)

SELECT
  commits_default_branch.project as project,
  commits_default_branch.hash_value as hash_value,
  branch_count
FROM `project_sample_default_branch.tr_sample_commits_default_branch_filtered`
AS commits_default_branch
LEFT JOIN (
  SELECT project, hash_value, COUNT(branch) AS branch_count
  FROM `project_sample.tr_sample_commits_filtered`
  GROUP BY project, hash_value
) AS commits_branch_count
ON commits_default_branch.project = commits_branch_count.project
  AND commits_default_branch.hash_value = commits_branch_count.hash_value;
=> project_sample_default_branch.tr_sample_commits_default_branch_filtered_branch_count


SELECT COUNT(*)
FROM (
  SELECT project, hash_value
  FROM `project_sample_default_branch.tr_sample_commits_default_branch_filtered_branch_count`
  GROUP BY project, hash_value
);
=> 486,419

SELECT COUNT(*)
FROM (
  SELECT project, hash_value
  FROM `project_sample_default_branch.tr_sample_commits_default_branch_filtered_branch_count`
  WHERE branch_count>1
  GROUP BY project, hash_value
);
=> 432,895

--- OBSERVATION: 89,00% of the commits to the default branch are also present in another branch, 

SELECT COUNT(*)
FROM (
  SELECT project, hash_value
  FROM `project_sample_default_branch.tr_sample_commits_default_branch_filtered_branch_count`
  WHERE branch_count=1
  GROUP BY project, hash_value
);
=> 53,524

--- OBSERVATION: only 11,00% of the commits to the default branch seem to be direct commits to the default branch


SELECT
  commits_default_branch.project as project,
  commits_default_branch.hash_value as hash_value,
  branch_count
FROM `project_sample_default_branch.tr_sample_commits_default_branch_before_ci`
AS commits_default_branch
LEFT JOIN (
  SELECT project, hash_value, COUNT(branch) AS branch_count
  FROM `project_sample.tr_sample_commits_filtered`
  GROUP BY project, hash_value
) AS commits_branch_count
ON commits_default_branch.project = commits_branch_count.project
  AND commits_default_branch.hash_value = commits_branch_count.hash_value;
=> project_sample_default_branch.tr_sample_commits_default_branch_before_ci_branch_count

SELECT COUNT(*)
FROM (
  SELECT project, hash_value
  FROM `project_sample_default_branch.tr_sample_commits_default_branch_before_ci_branch_count`
  GROUP BY project, hash_value
);
=> 70,285

SELECT COUNT(*)
FROM (
  SELECT project, hash_value
  FROM `project_sample_default_branch.tr_sample_commits_default_branch_before_ci_branch_count`
  WHERE branch_count>1
  GROUP BY project, hash_value
);
=> 62,590

--- OBSERVATION: 89,05% of the commits to the default branch before ci are also present in another branch, 

SELECT COUNT(*)
FROM (
  SELECT project, hash_value
  FROM `project_sample_default_branch.tr_sample_commits_default_branch_before_ci_branch_count`
  WHERE branch_count=1
  GROUP BY project, hash_value
);
=> 7,695

--- OBSERVATION: only 10,95% of the commits to the default branch before ci seem to be direct commits to the default branch


SELECT
  commits_default_branch.project as project,
  commits_default_branch.hash_value as hash_value,
  branch_count
FROM `project_sample_default_branch.tr_sample_commits_default_branch_during_ci`
AS commits_default_branch
LEFT JOIN (
  SELECT project, hash_value, COUNT(branch) AS branch_count
  FROM `project_sample.tr_sample_commits_filtered`
  GROUP BY project, hash_value
) AS commits_branch_count
ON commits_default_branch.project = commits_branch_count.project
  AND commits_default_branch.hash_value = commits_branch_count.hash_value;
=> project_sample_default_branch.tr_sample_commits_default_branch_during_ci_branch_count

SELECT COUNT(*)
FROM (
  SELECT project, hash_value
  FROM `project_sample_default_branch.tr_sample_commits_default_branch_during_ci_branch_count`
  GROUP BY project, hash_value
);
=> 70,285

SELECT COUNT(*)
FROM (
  SELECT project, hash_value
  FROM `project_sample_default_branch.tr_sample_commits_default_branch_during_ci_branch_count`
  WHERE branch_count>1
  GROUP BY project, hash_value
);
=> 62,590

--- OBSERVATION: 89,05% of the commits to the default branch before ci are also present in another branch, 

SELECT COUNT(*)
FROM (
  SELECT project, hash_value
  FROM `project_sample_default_branch.tr_sample_commits_default_branch_during_ci_branch_count`
  WHERE branch_count=1
  GROUP BY project, hash_value
);
=> 7,695

--- OBSERVATION: only 10,95% of the commits to the default branch before ci seem to be direct commits to the default branch
