#setwd("/Users/sebastian/git/projects/ci/data-retrieval")
setwd("F:/Git/projects/ci/data-retrieval")

# load global function
source("../global.R")

# load local functions
source("functions.R")

# read project information
data <- read.csv("3_last_build_vs_activity.csv", header=TRUE, sep=",", quote="\"", na.strings=c(""), stringsAsFactors=FALSE)
nrow(data)
# 487

# remove projects with missing data
data <- data[!is.na(data$last_build_vs_activity),]
nrow(data)
# 485

boxplot(data$last_build_vs_activity)

d <- density(data$last_build_vs_activity)
plot(d)

summary(data$last_build_vs_activity)
# Min.  1st Qu.   Median     Mean  3rd Qu.     Max. 
# -824.000    0.000    0.000   -7.495   10.000 1776.000 

# outliers
thresholds <- outlier_thresholds(data$last_build_vs_activity, 1.5)
thresholds$lower
# -15
thresholds$upper
# 25

# filter data
data_filtered <- data[data$last_build_vs_activity>=thresholds$lower & data$last_build_vs_activity<=thresholds$upper,]
nrow(data_filtered)
# 324

# export filtered projects
write.table(data_filtered$project, file="3_projects_filtered_build_activity.csv", sep=";", col.names=FALSE, row.names=FALSE, na="", quote=TRUE, qmethod="double", fileEncoding="UTF-8")


# plot data
pdf("3_last_build_vs_activity.pdf", width=12, height=12)
par(
  bg="white",
  #mar = c(1.2, 1.0, 0.1, 0.0)+0.1, # subplot margins (bottom, left, top, right)
  #omi = c(0.2, 0.5, 0.0, 0.0)+0.1,  # outer margins in inches (bottom, left, top, right)
  mfrow = c(1, 1),
  #pin = (width, height)
  #mfcol # draw in columns
  # increase font size
  cex=1.4,
  cex.main=1.4,
  cex.sub=1.4,
  cex.lab=1.2,
  cex.axis=1.2
)
#layout(matrix(c(1,2,3,4,5,5), 3, 2, byrow = TRUE))
#layout(2, 1)

# colors with alpha channel
library(grDevices)
gray_transparent <- adjustcolor("gray60", alpha.f=0.5)
gray_lighter <- adjustcolor("gray85", alpha.f=1)
gray_light <- adjustcolor("gray60", alpha.f=1)
gray_dark <- adjustcolor("gray34", alpha.f=1)
gray_darker <- adjustcolor("gray24", alpha.f=1)
blue_light <- adjustcolor("skyblue1", alpha.f=1)
blue_dark <- adjustcolor("steelblue4", alpha.f=1)
red_light <- adjustcolor("salmon", alpha.f=1)
red_dark <- adjustcolor("firebrick3", alpha.f=1)

# histogram
hist(data$last_build_vs_activity, 
     main="Days between last build and last commit/merge on default branch", 
     freq=TRUE,
     xlab="< last build before last commit | last build after last commit >",
     ylab="Number of projects",
     border="white",
     col="white",
     #labels=c(rep("", 10), "Selected"),
     xlim=c(-150,150),
     ylim=c(0, 350),
     breaks=seq(-875, 1900, by=50),
     xaxt="n",
     yaxt="n"
)
for (y in seq(0, 350, by=50)) {
  segments(x0=-150, y0=y, x1=150, y1=y, lty=1, lwd=1, col=gray_lighter)
}
hist(data$last_build_vs_activity,
     add=TRUE,
     main="", 
     freq=TRUE,
     xlab="",
     ylab="Number of projects",
     border=gray_dark,
     col=gray_lighter,
     #labels=c(rep("", 10), "Selected"),
     xlim=c(-150,150),
     ylim=c(0, 350),
     breaks=seq(-875, 1900, by=50),
     xaxt="n",
     yaxt="n"
)
library(beanplot)
beanplot(data$last_build_vs_activity,
         add=TRUE,
         main="",
         ylim=c(0, 350),
         what=c(0,1,0,0),
         border=gray_dark,
         log="",
         col=c(gray_transparent, gray_transparent, gray_transparent, gray_transparent),
         horizontal=TRUE,
         side="second",
         boxwex=800,
         axes=FALSE
         #overallline="mean"
)
#abline(v=median(data_commits$last_build_vs_activity), lty=2, col="blue")
abline(v=thresholds$lower+1.62, lty=3, lwd=2, col=gray_dark)
abline(v=thresholds$upper+0.62, lty=3, lwd=2, col=gray_dark)  
boxplot(data$last_build_vs_activity,
        add=TRUE,
        outline=FALSE,
        horizontal=TRUE,
        ylim=c(0,350),
        log="",
        col="white",
        # https://stackoverflow.com/a/28890111
        lwd=2,
        medlwd=2,
        #staplelty=0,
        whisklty=1,
        #staplelty=0,
        whiskcol=gray_darker,
        medcol=gray_darker,
        boxcol=gray_darker,
        staplecol=gray_darker,
        boxwex=22,
        axes=FALSE
)
axis(1, at=seq(-150, 150, by=50), labels=seq(-150, 150, by=50))
axis(2, at=seq(0, 350, by=50), labels=seq(0, 350, by=50), las=2)

#par(mfrow = c(1, 1))
dev.off()
