--- Status: 19.07.2017


--- get commits to default branch up to one year before first build
SELECT  
  c.project as project,
  branch,
  hash_value,
  author_name,
  author_email,
  author_date,
  commit_name,
  commit_email,
  commit_date,
  log_message_length,
  file_count,
  lines_added,
  lines_deleted,
  file_extensions
FROM `project_sample_default_branch.tr_sample_commits_default_branch` as c
INNER JOIN `project_sample_default_branch.tr_sample_default_branch_activity_filtered` as p
ON c.project = p.project
  AND DATE_DIFF(CAST(first_build AS DATE), CAST(commit_date AS DATE), DAY) > 0
  AND DATE_DIFF(CAST(first_build AS DATE), CAST(commit_date AS DATE), DAY) <= 365;
=> project_sample_default_branch.tr_sample_commits_default_branch_before_ci


--- get merges into default branch up to one year before first build
SELECT  
  m.project as project,
  branch,
  hash_value,
  merged_commits,
  author_name,
  author_email,
  author_date,
  commit_name,
  commit_email,
  commit_date,
  log_message_length,
  file_count,
  lines_added,
  lines_deleted,
  file_extensions,
  pull_request_id,
  source_user,
  source_branch
FROM `project_sample_default_branch.tr_sample_merges_default_branch` as m
INNER JOIN `project_sample_default_branch.tr_sample_default_branch_activity_filtered` as p
ON m.project = p.project
  AND DATE_DIFF(CAST(first_build AS DATE), CAST(commit_date AS DATE), DAY) > 0
  AND DATE_DIFF(CAST(first_build AS DATE), CAST(commit_date AS DATE), DAY) <= 365;
=> project_sample_default_branch.tr_sample_merges_default_branch_before_ci


--- get unique projects
SELECT DISTINCT project FROM `project_sample_default_branch.tr_sample_commits_default_branch_before_ci`
UNION DISTINCT
SELECT DISTINCT project FROM `project_sample_default_branch.tr_sample_merges_default_branch_before_ci`;
--- 321 before and after filtering


--- get projects without activity on default branch before ci
SELECT p.project as project, pb.project as project_before_ci
FROM (
  SELECT DISTINCT project
  FROM `project_sample_default_branch.tr_sample_default_branch_activity_filtered`
) as p
LEFT JOIN (
  SELECT DISTINCT project FROM `project_sample_default_branch.tr_sample_commits_default_branch_before_ci`
  UNION DISTINCT
  SELECT DISTINCT project FROM `project_sample_default_branch.tr_sample_merges_default_branch_before_ci`
) as pb
ON p.project = pb.project
WHERE pb.project is null;

--- none found after filtering, before:
--- DECISION: Exclude those projects (no activity on default branch before ci):
--- total: 3


--- get commits to default branch up to one year after first build, but not after last build
SELECT  
  c.project as project,
  branch,
  hash_value,
  author_name,
  author_email,
  author_date,
  commit_name,
  commit_email,
  commit_date,
  log_message_length,
  file_count,
  lines_added,
  lines_deleted,
  file_extensions
FROM `project_sample_default_branch.tr_sample_commits_default_branch` as c
INNER JOIN `project_sample_default_branch.tr_sample_default_branch_activity_filtered` as p
ON c.project = p.project
  AND DATE_DIFF(CAST(commit_date AS DATE), CAST(first_build AS DATE), DAY) >= 0
  AND DATE_DIFF(CAST(commit_date AS DATE), CAST(first_build AS DATE), DAY) <= 365
  AND DATE_DIFF(CAST(commit_date AS DATE), CAST(last_build AS DATE), DAY) <= 0;
=> project_sample_default_branch.tr_sample_commits_default_branch_during_ci


--- get merges into default branch up to one year after first build, but not after last build
SELECT  
  m.project as project,
  branch,
  hash_value,
  merged_commits,
  author_name,
  author_email,
  author_date,
  commit_name,
  commit_email,
  commit_date,
  log_message_length,
  file_count,
  lines_added,
  lines_deleted,
  file_extensions,
  pull_request_id,
  source_user,
  source_branch
FROM `project_sample_default_branch.tr_sample_merges_default_branch` as m
INNER JOIN `project_sample_default_branch.tr_sample_default_branch_activity_filtered` as p
ON m.project = p.project
  AND DATE_DIFF(CAST(commit_date AS DATE), CAST(first_build AS DATE), DAY) >= 0
  AND DATE_DIFF(CAST(commit_date AS DATE), CAST(first_build AS DATE), DAY) <= 365
  AND DATE_DIFF(CAST(commit_date AS DATE), CAST(last_build AS DATE), DAY) <= 0;
=> project_sample_default_branch.tr_sample_merges_default_branch_during_ci
  
  
--- get unique projects
SELECT DISTINCT project FROM `project_sample_default_branch.tr_sample_commits_default_branch_during_ci`
UNION DISTINCT
SELECT DISTINCT project FROM `project_sample_default_branch.tr_sample_merges_default_branch_during_ci`;
--- 321 (324 before filtering)

--- get projects without activity on default branch during ci
SELECT p.project as project, pb.project as project_during_ci
FROM (
  SELECT DISTINCT project
  FROM `project_sample_default_branch.tr_sample_default_branch_activity_filtered`
) as p
LEFT JOIN (
  SELECT DISTINCT project FROM `project_sample_default_branch.tr_sample_commits_default_branch_during_ci`
  UNION DISTINCT
  SELECT DISTINCT project FROM `project_sample_default_branch.tr_sample_merges_default_branch_during_ci`
) as pb
ON p.project = pb.project
WHERE pb.project is null;

--- none found before and after filtering


--- Exclude further projects (only before ci relevant here because no matches during ci) (see "3_default_branch.sql" for previous filtering):
SELECT *
FROM `project_sample_default_branch.tr_sample_default_branch_activity_filtered`
WHERE project NOT IN (
  SELECT p.project as project
  FROM (
    SELECT DISTINCT project
    FROM `project_sample_default_branch.tr_sample_default_branch_activity_filtered`
  ) as p
  LEFT JOIN (
    SELECT DISTINCT project FROM `project_sample_default_branch.tr_sample_commits_default_branch_before_ci`
    UNION DISTINCT
    SELECT DISTINCT project FROM `project_sample_default_branch.tr_sample_merges_default_branch_before_ci`
  ) as pb
  ON p.project = pb.project
  WHERE pb.project is null
);
=> project_sample_default_branch.tr_sample_default_branch_activity_filtered_tmp
=> project_sample_default_branch.tr_sample_default_branch_activity_filtered

--- count number of projects after filtering
SELECT COUNT(DISTINCT project)
FROM `project_sample_default_branch.tr_sample_default_branch_activity_filtered`;
--- 321


--- filter commits to default branch
SELECT
  commits.project as project,
  branch,
  hash_value,
  author_name,
  author_email,
  author_date,
  commit_name,
  commit_email,
  commit_date,
  log_message_length,
  file_count,
  lines_added,
  lines_deleted,
  file_extensions
FROM `project_sample_default_branch.tr_sample_commits_default_branch` commits
INNER JOIN (
  SELECT DISTINCT project
  FROM `project_sample_default_branch.tr_sample_default_branch_activity_filtered`
) filtered_projects
ON commits.project = filtered_projects.project;
=> project_sample_default_branch.tr_sample_commits_default_branch_filtered (overwrite)


--- filter merges into default branch
SELECT
  merges.project as project,
  branch,
  hash_value,
  merged_commits,
  author_name,
  author_email,
  author_date,
  commit_name,
  commit_email,
  commit_date,
  log_message_length,
  file_count,
  lines_added,
  lines_deleted,
  file_extensions,
  pull_request_id,
  source_user,
  source_branch  
FROM `project_sample_default_branch.tr_sample_merges_default_branch` merges
INNER JOIN (
  SELECT DISTINCT project
  FROM `project_sample_default_branch.tr_sample_default_branch_activity_filtered`
) filtered_projects
ON merges.project = filtered_projects.project;
=> project_sample_default_branch.tr_sample_merges_default_branch_filtered (overwrite)


--- get number of remaining projects
SELECT COUNT(DISTINCT project) FROM `project_sample_default_branch.tr_sample_default_branch_activity_filtered`;
--- 321
SELECT COUNT(*) FROM (
  SELECT DISTINCT project FROM `project_sample_default_branch.tr_sample_commits_default_branch_filtered`
  UNION DISTINCT
  SELECT DISTINCT project FROM `project_sample_default_branch.tr_sample_merges_default_branch_filtered`
 );
--- 321


--- recreate before/during tables
--- see above (start again with first query)


--- create filtered projects sample table
WITH
  project_lang AS (
    SELECT gh_project_name, gh_lang
    FROM `msr-mining-challenge.travistorrent.travistorrent_11_1_2017`
    GROUP BY gh_project_name, gh_lang
  ),
  projects AS (
    SELECT
      REPLACE(gh_project_name, "/", "_") as project,
      gh_project_name,
      gh_owner_id,
      gh_project_id,
      tr_first_build,
      tr_last_build,
      gh_first_commit,
      gh_last_commit,
      tr_timespan_before_ci,
      tr_timespan_during_ci,
      tr_timespan_after_ci,
      tr_build_count,
      gh_commit_count,
      gh_commit_count_before_ci,
      gh_commit_count_during_ci,
      gh_commit_count_after_ci,
      default_branch
    FROM
      `travistorrent.tr_projects_sample` as projects_sample
    LEFT JOIN (
      SELECT DISTINCT project
      FROM `project_sample_default_branch.tr_sample_default_branch_activity_filtered`
    ) as projects_filtered
    ON REPLACE(projects_sample.gh_project_name, "/", "_") = projects_filtered.project
    WHERE projects_filtered.project is not null
  )
SELECT
  project,
  projects.gh_project_name AS gh_project_name,
  gh_lang,
  gh_owner_id,
  gh_project_id,
  tr_first_build,
  tr_last_build,
  gh_first_commit,
  gh_last_commit,
  tr_timespan_before_ci,
  tr_timespan_during_ci,
  tr_timespan_after_ci,
  tr_build_count,
  gh_commit_count,
  gh_commit_count_before_ci,
  gh_commit_count_during_ci,
  gh_commit_count_after_ci,
  default_branch
FROM projects
LEFT JOIN project_lang
ON projects.gh_project_name = project_lang.gh_project_name
ORDER BY project ASC;

=> travistorrent.tr_projects_sample_filtered


# only consider projects which used GH (almost) from the beginning
SELECT 
  project,
  gh_project_name,
  gh_lang,
  gh_owner_id,
  gh_project_id,
  tr_first_build,
  tr_last_build,
  gh_first_commit,
  gh_last_commit,
  default_branch
FROM `travistorrent.tr_projects_sample_filtered` s
JOIN `ghtorrent-bq.ght.projects` p
ON s.gh_project_id = p.id
WHERE abs(date_diff(date(s.gh_first_commit), date(p.created_at), DAY)) <= 7;

=> travistorrent.tr_projects_sample_filtered_2

--- count number of projects after filtering
SELECT COUNT(DISTINCT project)
FROM `travistorrent.tr_projects_sample_filtered_2`;
--- 113


--- filter commits to default branch
SELECT
  commits.project as project,
  branch,
  hash_value,
  author_name,
  author_email,
  author_date,
  commit_name,
  commit_email,
  commit_date,
  log_message_length,
  file_count,
  lines_added,
  lines_deleted,
  file_extensions
FROM `project_sample_default_branch.tr_sample_commits_default_branch_before_ci` commits
INNER JOIN `travistorrent.tr_projects_sample_filtered_2` filtered_projects
ON commits.project = filtered_projects.project;

=> project_sample_default_branch_2.tr_sample_commits_default_branch_before_ci

SELECT
  commits.project as project,
  branch,
  hash_value,
  author_name,
  author_email,
  author_date,
  commit_name,
  commit_email,
  commit_date,
  log_message_length,
  file_count,
  lines_added,
  lines_deleted,
  file_extensions
FROM `project_sample_default_branch.tr_sample_commits_default_branch_during_ci` commits
INNER JOIN `travistorrent.tr_projects_sample_filtered_2` filtered_projects
ON commits.project = filtered_projects.project;

=> project_sample_default_branch_2.tr_sample_commits_default_branch_during_ci


--- filter merges into default branch
SELECT
  merges.project as project,
  branch,
  hash_value,
  merged_commits,
  author_name,
  author_email,
  author_date,
  commit_name,
  commit_email,
  commit_date,
  log_message_length,
  file_count,
  lines_added,
  lines_deleted,
  file_extensions,
  pull_request_id,
  source_user,
  source_branch  
FROM `project_sample_default_branch.tr_sample_merges_default_branch_before_ci` merges
INNER JOIN `travistorrent.tr_projects_sample_filtered_2` filtered_projects
ON merges.project = filtered_projects.project;

=> project_sample_default_branch_2.tr_sample_merges_default_branch_before_ci

SELECT
  merges.project as project,
  branch,
  hash_value,
  merged_commits,
  author_name,
  author_email,
  author_date,
  commit_name,
  commit_email,
  commit_date,
  log_message_length,
  file_count,
  lines_added,
  lines_deleted,
  file_extensions,
  pull_request_id,
  source_user,
  source_branch  
FROM `project_sample_default_branch.tr_sample_merges_default_branch_during_ci` merges
INNER JOIN `travistorrent.tr_projects_sample_filtered_2` filtered_projects
ON merges.project = filtered_projects.project;

=> project_sample_default_branch_2.tr_sample_merges_default_branch_during_ci


# how many build have been triggered by commits on the default branch?
with
  default_branch as (
    select
      projects.gh_project_name as gh_project_name,
      count(distinct tr_build_id) as build_count_default_branch
    from `travistorrent.travistorrent_11_1_2017` travis
    join `travistorrent.tr_projects_sample_filtered_2` projects
    on travis.gh_project_name = projects.gh_project_name
     and travis.git_branch = projects.default_branch
    group by gh_project_name
  ),
  others as (
    select
      projects.gh_project_name as gh_project_name,
      count(distinct tr_build_id) as build_count_other_branches
    from `travistorrent.travistorrent_11_1_2017` travis
    join `travistorrent.tr_projects_sample_filtered_2` projects
    on travis.gh_project_name = projects.gh_project_name
     and travis.git_branch <> projects.default_branch
    group by gh_project_name
  )
select
  default_branch.gh_project_name as gh_project_name,
  build_count_default_branch,
  build_count_other_branches,
  build_count_default_branch / (build_count_default_branch + build_count_other_branches) as default_branch_ratio
from default_branch
join others
on default_branch.gh_project_name = others.gh_project_name;

=> travistorrent.tr_projects_sample_filtered_2_branch_builds
