# status: 06.02.2018 

# retrieve sample to compare selected projects to
with
  commits as (select project_id, min(created_at) as created_at_min, max(created_at) as created_at_max
  from `ghtorrent-bq.ght.commits`
  group by project_id),
  projects as (select id as project_id, owner_id, name, language, created_at
  from `ghtorrent-bq.ght.projects`
  where forked_from is null
    and deleted is false
    and (language = 'Java' or language = 'Ruby')),
  users as (select id as user_id, login
  from `ghtorrent-bq.ght.users`),
  watchers as (select repo_id as project_id, count(distinct user_id) as watcher_count
  from `ghtorrent-bq.ght.watchers`
  group by repo_id)
select projects_sample_2.project_id as project_id, owner_id, repo_name, language, created_at, first_commit, mid_date, last_commit, watcher_count
from (
  select project_id, owner_id, concat(users.login, "/", projects_sample.name) as repo_name, language, created_at, first_commit, mid_date, last_commit
  from (
    select projects.project_id as project_id, owner_id, name, language, projects.created_at as created_at, commits.created_at_min as first_commit, commits.created_at_max as last_commit,
    # date inbetween the first and last commit
    date_add(date(commits.created_at_min), interval CAST((CAST(abs(date_diff(date(commits.created_at_min), date(commits.created_at_max), DAY)) as INT64)/2) AS INT64) DAY) as mid_date
    from projects
    join commits
    on projects.project_id = commits.project_id
    where abs(date_diff(date(projects.created_at), date(commits.created_at_min), DAY)) <= 7 # developed on GitHub from the beginning
      and abs(date_diff(date(commits.created_at_min), date(commits.created_at_max), DAY)) >= 730 # developed for at least two years
  ) as projects_sample
  join users
  on projects_sample.owner_id = users.user_id
) as projects_sample_2
join watchers
on projects_sample_2.project_id = watchers.project_id
where watcher_count >= 10;

=> comparison_sample.project_sample

# 1. Export to comparison_project_sample.csv
# 2. Use R-script "draw_comparison_sample.R" to draw random sample from those projects
# 3. Retrieve and parse git logs for the sampled projects
# 4. Import branches, commits, and merges into BigQuery (dataset "comparison_sample")

# status: 17.06.2018 

--- filter commits to default branch 
SELECT
  c.project as project,
  c.branch as branch,
  hash_value,
  author_name,
  author_email,
  author_date,
  commit_name,
  commit_email,
  commit_date,
  log_message_length,
  file_count,
  lines_added,
  lines_deleted,
  file_extensions
FROM `comparison_sample.commits` as c
JOIN (
  SELECT project, branch as default_branch
  FROM `comparison_sample.branches`
  WHERE default_branch=true
) as b
ON c.project = REPLACE(b.project, '/', '_')
  AND c.branch = b.default_branch;
  
=> comparison_sample.commits_default_branch


--- filter merges into default branch 
SELECT
  m.project as project,
  m.branch as branch,
  hash_value,
  merged_commits,
  author_name,
  author_email,
  author_date,
  commit_name,
  commit_email,
  commit_date,
  log_message_length,
  file_count,
  lines_added,
  lines_deleted,
  file_extensions,
  pull_request_id,
  source_user,
  source_branch
FROM `comparison_sample.merges` as m
JOIN (
  SELECT project, branch as default_branch
  FROM `comparison_sample.branches`
  WHERE default_branch=true
) as b
ON m.project = REPLACE(b.project, '/', '_')
  AND m.branch = b.default_branch;
  
=> comparison_sample.merges_default_branch


--- get commits to default branch up to one year before mid date
SELECT  
  c.project as project,
  branch,
  hash_value,
  author_name,
  author_email,
  author_date,
  commit_name,
  commit_email,
  commit_date,
  log_message_length,
  file_count,
  lines_added,
  lines_deleted,
  file_extensions
FROM `comparison_sample.commits_default_branch` as c
INNER JOIN `comparison_sample.project_sample` as p
ON c.project = REPLACE(p.repo_name, '/', '_')
  AND DATE_DIFF(mid_date, CAST(commit_date AS DATE), DAY) > 0
  AND DATE_DIFF(mid_date, CAST(commit_date AS DATE), DAY) <= 365;
  
=> comparison_sample.commits_default_branch_before_mid


--- get merges into default branch up to one year before first build
SELECT  
  m.project as project,
  branch,
  hash_value,
  merged_commits,
  author_name,
  author_email,
  author_date,
  commit_name,
  commit_email,
  commit_date,
  log_message_length,
  file_count,
  lines_added,
  lines_deleted,
  file_extensions,
  pull_request_id,
  source_user,
  source_branch
FROM `comparison_sample.merges_default_branch` as m
INNER JOIN `comparison_sample.project_sample` as p
ON m.project = REPLACE(p.repo_name, '/', '_')
  AND DATE_DIFF(mid_date, CAST(commit_date AS DATE), DAY) > 0
  AND DATE_DIFF(mid_date, CAST(commit_date AS DATE), DAY) <= 365;
  
=> project_sample_default_branch.merges_default_branch_before_mid


--- get commits to default branch up to one year after mid date
SELECT  
  c.project as project,
  branch,
  hash_value,
  author_name,
  author_email,
  author_date,
  commit_name,
  commit_email,
  commit_date,
  log_message_length,
  file_count,
  lines_added,
  lines_deleted,
  file_extensions
FROM `comparison_sample.commits_default_branch` as c
INNER JOIN `comparison_sample.project_sample` as p
ON c.project = REPLACE(p.repo_name, '/', '_')
  AND DATE_DIFF(CAST(commit_date AS DATE), mid_date, DAY) >= 0
  AND DATE_DIFF(CAST(commit_date AS DATE), mid_date, DAY) <= 365;
  
=> comparison_sample.commits_default_branch_after_mid


--- get merges into default branch up to one year after first build
SELECT  
  m.project as project,
  branch,
  hash_value,
  merged_commits,
  author_name,
  author_email,
  author_date,
  commit_name,
  commit_email,
  commit_date,
  log_message_length,
  file_count,
  lines_added,
  lines_deleted,
  file_extensions,
  pull_request_id,
  source_user,
  source_branch
FROM `comparison_sample.merges_default_branch` as m
INNER JOIN `comparison_sample.project_sample` as p
ON m.project = REPLACE(p.repo_name, '/', '_')
  AND DATE_DIFF(CAST(commit_date AS DATE), mid_date, DAY) >= 0
  AND DATE_DIFF(CAST(commit_date AS DATE), mid_date, DAY) <= 365;
  
=> project_sample_default_branch.merges_default_branch_after_mid
